//******************************************************************************
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************

#define HAL_I2C_MODULE_ENABLED
#define HAL_GPIO_MODULE_ENABLED
#include "lps25hb_i2c_lib.h"
#include "stm32f1xx_hal.h"


I2C_HandleTypeDef hi2c_LPS25;

//LPS25 hardware initialization for I2C operation
void LPS25_I2C_HwConfig(void)
{

  GPIO_InitTypeDef  GPIO_InitStruct;

  //clocks activation
  if (LPS25_I2C == I2C1) __HAL_RCC_I2C1_CLK_ENABLE(); else
  if (LPS25_I2C == I2C2) __HAL_RCC_I2C2_CLK_ENABLE();

  if (LPS25_I2C_GPIO == GPIOA) __HAL_RCC_GPIOA_CLK_ENABLE(); else
  if (LPS25_I2C_GPIO == GPIOB) __HAL_RCC_GPIOB_CLK_ENABLE(); else
  if (LPS25_I2C_GPIO == GPIOC) __HAL_RCC_GPIOC_CLK_ENABLE(); else
  if (LPS25_I2C_GPIO == GPIOD) __HAL_RCC_GPIOD_CLK_ENABLE(); else
  if (LPS25_I2C_GPIO == GPIOE) __HAL_RCC_GPIOE_CLK_ENABLE();

  //GPIO Config
  GPIO_InitStruct.Pin = LPS25_SCL | LPS25_SDA;      //I2C - SCL & SDA
  GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(LPS25_I2C_GPIO, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = LPS25_SA0|LPS25_CS;         //I2C SA0 & CS
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LPS25_I2C_GPIO, &GPIO_InitStruct);
  if (LPS25_I2C_Addr==0xB8) {HAL_GPIO_WritePin(LPS25_I2C_GPIO, LPS25_SA0, GPIO_PIN_RESET);}
  else {HAL_GPIO_WritePin(LPS25_I2C_GPIO, LPS25_SA0, GPIO_PIN_SET);}
  HAL_GPIO_WritePin(LPS25_I2C_GPIO, LPS25_CS, GPIO_PIN_SET);  //set CS high to work in I2C mode


  //I2C Config
  hi2c_LPS25.Instance = LPS25_I2C;
  hi2c_LPS25.Init.ClockSpeed = 100000;
  hi2c_LPS25.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c_LPS25.Init.OwnAddress1 = 0;
  hi2c_LPS25.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c_LPS25.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c_LPS25.Init.OwnAddress2 = 0;
  hi2c_LPS25.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c_LPS25.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  HAL_I2C_Init(&hi2c_LPS25);
}


//LPS25 initialization
char LPS25_I2C_Init(LPS25_ODR_TypeDef ODR_Mode)
{
  unsigned char RegVal=0, LPS25Settings;

  //soft reset LPS25
  LPS25_I2C_WriteRegister(LPS25_CTRL_REG2, LPS25_CTRL_REG2_SWRESET);
  HAL_Delay(100);

  //PowerUp and write settings
  LPS25Settings = LPS25_CTRL_REG1_PD_Active | ODR_Mode | LPS25_CTRL_REG1_BDU;
  LPS25_I2C_WriteRegister(LPS25_CTRL_REG1, LPS25Settings);

  //Read LPS25 self identifier - if 0xBD read back, LPS25 is up and running
  LPS25_I2C_ReadRegister(LPS25_WHO_AM_I, &RegVal);
  if (RegVal == 0xBD) return LPS25_OK;
  else return LPS25_ERROR;
}


//Reads one register value from LPS25
void LPS25_I2C_ReadRegister(unsigned char addr, unsigned char *v)
{
  HAL_I2C_Mem_Read(&hi2c_LPS25, LPS25_I2C_Addr, addr, 1, v, 1, 50);
}


//Writes one register value to LPS25
void LPS25_I2C_WriteRegister(unsigned char addr, unsigned char v)
{
  HAL_I2C_Mem_Write(&hi2c_LPS25, LPS25_I2C_Addr, addr, 1, &v, 1, 50);
}


//Gets pressure value from LPS25
//Pressure value as raw data, needs to be divided by 4096 to get real value
void LPS25_I2C_GetPress(int* Pressure)
{
  unsigned char Buffer[3]={0}; //XL,L,H

  //Read 3 consecutive registers in autoincrement mode starting from LPS25_PRESS_OUT_XL
  HAL_I2C_Mem_Read(&hi2c_LPS25, LPS25_I2C_Addr, LPS25_PRESS_OUT_XL|LPS25_ADDR_INC_I2C, 1, Buffer, 3, 50);

  *Pressure=(int)(((unsigned int)Buffer[2]<<16)+((unsigned int)Buffer[1]<<8)+
		  ((unsigned int)Buffer[0]));
  if(Buffer[2] & 0x80){*Pressure |= 0xFF000000; } //correction of upper bits of 32-bit value when pressure is negative to maintain proper 2's complement handling
}

//Gets temperature value from LPS25
//Temperature value as raw data, real value needs to be calulated as 42.5+raw/480.
//Temp sensor used internaly for correcting preassure readout.
//Not intended for accurate temperature meassurement. Low accuracy!
void LPS25_I2C_GetTemp(short int* Temperature)
{
  unsigned char Buffer[2]={0}; //L,H

  //Read 3 consecutive registers in autoincrement mode starting from LPS25_TEMP_OUT_L
  HAL_I2C_Mem_Read(&hi2c_LPS25, LPS25_I2C_Addr, LPS25_TEMP_OUT_L|LPS25_ADDR_INC_I2C, 1, Buffer, 2, 50);

  *Temperature=(short int)(((unsigned short int)Buffer[1]<<8)+((unsigned short int)Buffer[0]));
}

//Sets LPS25 FIFO mode
//Sets FIFO mode and watermark level (or mean calculation length in FIFO Mean mode)
//Enables (or disables) FIFO according to selected mode
void LPS25_I2C_SetFIFOMode(LPS25_FIFOMode_TypeDef Mode, LPS25_FIFOWatermarkMean_TypeDef WtmMean)
{
  unsigned char prevSettings, newSettings;

  LPS25_I2C_ReadRegister(LPS25_CTRL_REG2, &prevSettings);
  LPS25_I2C_WriteRegister(LPS25_FIFO_CTRL, Mode|WtmMean); //Set FIFO mode and watermark (or mean length in FIFO Mean mode)
  if (Mode!= Bypass){
	newSettings=prevSettings|LPS25_CTRL_REG2_FIFO_EN;    //Enable FIFO for all FIFO modes except Bypass
  } else {
	newSettings=prevSettings&(~LPS25_CTRL_REG2_FIFO_EN); //Disable FIFO
  }
  LPS25_I2C_WriteRegister(LPS25_CTRL_REG2, newSettings);
}


//Enable or disable LPS25 Autozero
//parameters: Action - LPS25_ENABLE, LPS25_DISABLE
void LPS25_I2C_Autozero(LPS25_bool Action)
{
	unsigned char prevSettings, newSettings;

	if (Action==LPS25_ENABLE){
      LPS25_I2C_ReadRegister(LPS25_CTRL_REG2, &prevSettings);
	  newSettings=prevSettings|LPS25_CTRL_REG2_AUTOZERO;    //Enable Autozero function
      LPS25_I2C_WriteRegister(LPS25_CTRL_REG2, newSettings);
	} else {
	  LPS25_I2C_ReadRegister(LPS25_CTRL_REG1, &prevSettings);
      newSettings=prevSettings|LPS25_CTRL_REG1_RESET_AZ;    //Disable Autozero function
	  LPS25_I2C_WriteRegister(LPS25_CTRL_REG1, newSettings);
	}
}


//Perform LPS25 one measurement in OneShot mode
void LPS25_I2C_DoOneShot(void)
{
	unsigned char prevSettings, newSettings;

    LPS25_I2C_ReadRegister(LPS25_CTRL_REG2, &prevSettings);
    newSettings=prevSettings|LPS25_CTRL_REG2_ONE_SHOT;    //Enable OneShot bit -> perform one measurement
    LPS25_I2C_WriteRegister(LPS25_CTRL_REG2, newSettings);
}
