//******************************************************************************
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************

#define HAL_SPI_MODULE_ENABLED
#define HAL_GPIO_MODULE_ENABLED
#include "lps25hb_spi_lib.h"
#include "stm32f1xx_hal.h"


SPI_HandleTypeDef hspi_LPS25;
LPS25_SPIMode_TypeDef gSPIMode;  //global SPIMode

//LPS25 hardware initialization for SPI operation
void LPS25_SPI_HwConfig(LPS25_SPIMode_TypeDef SPIMode)
{

  GPIO_InitTypeDef  GPIO_InitStruct;

  gSPIMode=SPIMode;

  //clocks activation
  if (LPS25_SPI == SPI1) __HAL_RCC_SPI1_CLK_ENABLE(); else
  if (LPS25_SPI == SPI2) __HAL_RCC_SPI2_CLK_ENABLE();

  if (LPS25_SPI_GPIO == GPIOA) __HAL_RCC_GPIOA_CLK_ENABLE(); else
  if (LPS25_SPI_GPIO == GPIOB) __HAL_RCC_GPIOB_CLK_ENABLE(); else
  if (LPS25_SPI_GPIO == GPIOC) __HAL_RCC_GPIOC_CLK_ENABLE(); else
  if (LPS25_SPI_GPIO == GPIOD) __HAL_RCC_GPIOD_CLK_ENABLE(); else
  if (LPS25_SPI_GPIO == GPIOE) __HAL_RCC_GPIOE_CLK_ENABLE();

  //GPIO Config
  GPIO_InitStruct.Pin = LPS25_SS;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(LPS25_SPI_GPIO, &GPIO_InitStruct);
  HAL_GPIO_WritePin(LPS25_SPI_GPIO, LPS25_SS, GPIO_PIN_SET); //Set NSS state high

  GPIO_InitStruct.Pin = LPS25_SCK|LPS25_MOSI;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(LPS25_SPI_GPIO, &GPIO_InitStruct);

  if (SPIMode==SPI4w){
	  GPIO_InitStruct.Pin = LPS25_MISO;
	  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	  GPIO_InitStruct.Pull = GPIO_NOPULL;
	  HAL_GPIO_Init(LPS25_SPI_GPIO, &GPIO_InitStruct);
  }

  //SPI Config
  hspi_LPS25.Instance = LPS25_SPI;
  hspi_LPS25.Init.Mode = SPI_MODE_MASTER;
  if (SPIMode==SPI4w){
    hspi_LPS25.Init.Direction = SPI_DIRECTION_2LINES;
  } else {
	hspi_LPS25.Init.Direction = SPI_DIRECTION_1LINE;
  }
  hspi_LPS25.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi_LPS25.Init.CLKPolarity = SPI_POLARITY_HIGH;
  hspi_LPS25.Init.CLKPhase = SPI_PHASE_2EDGE;
  hspi_LPS25.Init.NSS = SPI_NSS_SOFT;
  hspi_LPS25.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_16;
  hspi_LPS25.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi_LPS25.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi_LPS25.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi_LPS25.Init.CRCPolynomial = 10;
  HAL_SPI_Init(&hspi_LPS25);
}


//Set NSS line low, activate slave
void SPI_NSS_Enable(void)
{
	HAL_GPIO_WritePin(LPS25_SPI_GPIO, LPS25_SS, GPIO_PIN_RESET);
}


//Set NSS line high, deactivate slave
void SPI_NSS_Disable(void)
{
	HAL_GPIO_WritePin(LPS25_SPI_GPIO, LPS25_SS, GPIO_PIN_SET);
}


//LPS25 initialization
char LPS25_SPI_Init(LPS25_ODR_TypeDef ODRMode)
{
  unsigned char RegVal=0, LPS25Settings;

  //soft reset LPS25, enable SPI, disable I2C
  LPS25_SPI_WriteRegister(LPS25_CTRL_REG2, LPS25_CTRL_REG2_SWRESET);
  HAL_Delay(100);
  LPS25_SPI_WriteRegister(LPS25_CTRL_REG1, gSPIMode);
  LPS25_SPI_WriteRegister(LPS25_CTRL_REG2, LPS25_CTRL_REG2_I2C_DIS);

  //PowerUp and write settings
  LPS25Settings = LPS25_CTRL_REG1_PD_Active | ODRMode | LPS25_CTRL_REG1_BDU | gSPIMode;
  LPS25_SPI_WriteRegister(LPS25_CTRL_REG1, LPS25Settings);

  //Read LPS25 self identifier - if 0xBD read back, LPS25 is up and running
  LPS25_SPI_ReadRegister(LPS25_WHO_AM_I, &RegVal);
  if (RegVal == 0xBD) return LPS25_OK;
  else return LPS25_ERROR;
}


//Reads one register value from LPS25
void LPS25_SPI_ReadRegister(unsigned char addr, unsigned char *v)
{
	unsigned char DataReceived=0;
	unsigned char DataToTransmit;

	DataToTransmit=LPS25_READ|LPS25_ADDR_NO_INC|addr;

	SPI_NSS_Enable();
	HAL_SPI_Transmit(&hspi_LPS25, &DataToTransmit, 1, 50);
	if (gSPIMode==SPI3w) {HAL_SPI_Receive(&hspi_LPS25,  &DataReceived, 1, 50);}
	HAL_SPI_Receive(&hspi_LPS25,  &DataReceived, 1, 50);
	SPI_NSS_Disable();
	*v=DataReceived;
}


//Writes one register value to LPS25
void LPS25_SPI_WriteRegister(unsigned char addr, unsigned char v)
{
	unsigned char DataToTransmit[2];

	DataToTransmit[0]=LPS25_WRITE|LPS25_ADDR_NO_INC|addr;
	DataToTransmit[1]=v;

	SPI_NSS_Enable();
	HAL_SPI_Transmit(&hspi_LPS25, DataToTransmit, 2, 50);
	SPI_NSS_Disable();
}


//Gets pressure value from LPS25
//Pressure value as raw data, needs to be divided by 4096 to get real value
void LPS25_SPI_GetPress(int* Pressure)
{
	unsigned char DataReceived[3]={0};
	unsigned char DataToTransmit;

	//Read 3 consecutive registers in autoincrement mode starting from LPS25_PRESS_OUT_XL
	DataToTransmit=LPS25_READ|LPS25_ADDR_INC_SPI|LPS25_PRESS_OUT_XL;
	SPI_NSS_Enable();
    HAL_SPI_Transmit(&hspi_LPS25, &DataToTransmit, 1, 50);
    if (gSPIMode==SPI3w) {HAL_SPI_Receive(&hspi_LPS25, DataReceived, 1, 50);}
    HAL_SPI_Receive(&hspi_LPS25, DataReceived, 3, 50);
	SPI_NSS_Disable();

	*Pressure=0;
    *Pressure=(int)(((unsigned int)DataReceived[2]<<16)+((unsigned int)DataReceived[1]<<8)+((unsigned int)DataReceived[0]));
    if(DataReceived[2] & 0x80){*Pressure |= 0xFF000000; } //correction of upper bits of 32-bit value when pressure is negative to maintain proper 2's complement handling
}

//Gets temperature value from LPS25
//Temperature value as raw data, real value needs to be calculated as 42.5+raw/480.
//Temp sensor used internally for correcting pressure readout.
//Not intended for accurate temperature meassurement. Low accuracy!
void LPS25_SPI_GetTemp(short int* Temperature)
{
	unsigned char DataReceived[2]={0};
	unsigned char DataToTransmit;

	//Read 2 consecutive registers in autoincrement mode starting from LPS25_TEMP_OUT_L
	DataToTransmit=LPS25_READ|LPS25_ADDR_INC_SPI|LPS25_TEMP_OUT_L;
	SPI_NSS_Enable();
    HAL_SPI_Transmit(&hspi_LPS25, &DataToTransmit, 1, 50);
    if (gSPIMode==SPI3w) {HAL_SPI_Receive(&hspi_LPS25, DataReceived, 1, 50);}
    HAL_SPI_Receive(&hspi_LPS25, DataReceived, 2, 50);
	SPI_NSS_Disable();

	*Temperature=(short int)(((unsigned short int)DataReceived[1]<<8)+((unsigned short int)DataReceived[0]));
}

//Sets LPS25 FIFO mode
//Sets FIFO mode and watermark level (or mean calculation length in FIFO Mean mode)
//Enables (or disables) FIFO according to selected mode
void LPS25_SPI_SetFIFOMode(LPS25_FIFOMode_TypeDef Mode, LPS25_FIFOWatermarkMean_TypeDef WtmMean)
{
  unsigned char prevSettings, newSettings;

  LPS25_SPI_ReadRegister(LPS25_CTRL_REG2, &prevSettings);
  LPS25_SPI_WriteRegister(LPS25_FIFO_CTRL, Mode|WtmMean); //Set FIFO mode and watermark (or mean length in FIFO Mean mode)
  if (Mode!= Bypass){
	newSettings=prevSettings|LPS25_CTRL_REG2_FIFO_EN;    //Enable FIFO for all FIFO modes except Bypass
  } else {
	newSettings=prevSettings&(~LPS25_CTRL_REG2_FIFO_EN); //Disable FIFO
  }
  LPS25_SPI_WriteRegister(LPS25_CTRL_REG2, newSettings);
}


//Enable or disable LPS25 Autozero
//parameters: Action - LPS25_ENABLE, LPS25_DISABLE
void LPS25_SPI_Autozero(LPS25_bool Action)
{
	unsigned char prevSettings, newSettings;

	if (Action==LPS25_ENABLE){
      LPS25_SPI_ReadRegister(LPS25_CTRL_REG2, &prevSettings);
	  newSettings=prevSettings|LPS25_CTRL_REG2_AUTOZERO;    //Enable Autozero function
      LPS25_SPI_WriteRegister(LPS25_CTRL_REG2, newSettings);
	} else {
	  LPS25_SPI_ReadRegister(LPS25_CTRL_REG1, &prevSettings);
      newSettings=prevSettings|LPS25_CTRL_REG1_RESET_AZ;    //Disable Autozero function
	  LPS25_SPI_WriteRegister(LPS25_CTRL_REG1, newSettings);
	}
}


//Perform LPS25 one measurement in OneShot mode
void LPS25_SPI_DoOneShot(void)
{
	unsigned char prevSettings, newSettings;

    LPS25_SPI_ReadRegister(LPS25_CTRL_REG2, &prevSettings);
    newSettings=prevSettings|LPS25_CTRL_REG2_ONE_SHOT;    //Enable OneShot bit -> perform one measurement
    LPS25_SPI_WriteRegister(LPS25_CTRL_REG2, newSettings);
}
